using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.Serialization;

using Syncfusion.Drawing;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace TestDropdownRenderer
{
	public class DropDownUserCellModel : GridDropDownCellModel
	{
		internal DropDownUser ddUser;

		protected DropDownUserCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}

		public DropDownUserCellModel(GridModel grid, DropDownUser ddUser)
			: base(grid)
		{
			this.ddUser = ddUser;
			this.ButtonBarSize = new Size(18,15);
			 
		}
	
		//note that this method create our new derived renderer
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new DropDownUserCellRenderer(control, this);
		}
	}

	public class DropDownUserCellRenderer: GridDropDownCellRenderer
	{
		internal DropDownUser ddUser;

		public DropDownUserCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
			this.ddUser = ((DropDownUserCellModel)this.Model).ddUser;
			this.ddUser.Visible = false;
			this.AddButton(new GridCellComboBoxButton(this));
			this.DisableTextBox = true;

			//usercontrol save and cancel events...
			this.ddUser.SaveValues += new EventHandler(user_Save);
			this.ddUser.CancelValues += new EventHandler(user_Cancel);
			
		}
	
		protected  override void InitializeDropDownContainer()
		{
			base.InitializeDropDownContainer();

			if (this.DropDownContainer != null)
			{
				this.DropDownContainer.Controls.Add(this.ddUser);
			}
		}

		public override void DropDownContainerShowingDropDown(object sender, CancelEventArgs e)
		{
			GridCurrentCell cc = this.Grid.CurrentCell;

			Size size = ddUser.Size;
			GridCurrentCellShowingDropDownEventArgs ce = new GridCurrentCellShowingDropDownEventArgs(size);
			Grid.RaiseCurrentCellShowingDropDown(ce);
			if (ce.Cancel)
			{
				e.Cancel = true;
				return;
			}

			ddUser.Size = ce.Size;
			this.DropDownContainer.Size = ddUser.Size;
			this.ddUser.SetValuesFromString(this.TextBoxText);
			ddUser.Visible = true;
		}
		public override void DropDownContainerCloseDropDown(object sender, PopupClosedEventArgs e)  
		{
          
			if (e.PopupCloseType == PopupCloseType.Done)
			{
				if (this.NotifyCurrentCellChanging())
				{
					ControlValue = this.ddUser.GetValuesToString();
					this.NotifyCurrentCellChanged();
				}
			}
           
			Grid.InvalidateRange(GridRangeInfo.Cell(RowIndex, ColIndex), GridRangeOptions.MergeCoveredCells); // Merge all cells

			base.DropDownContainerCloseDropDown(sender, e);
		}

		private void user_Save(object sender, EventArgs e)
		{
			GridStyleInfo cell;
			string initialText = this.ControlText;

           
			this.Grid.CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Done);

			cell = this.Grid.Model[Grid.CurrentCell.RowIndex,Grid.CurrentCell.ColIndex];
            //to save the changes in the edited cell
            this.Grid.CurrentCell.EndEdit();
			if(this.Grid is GridControl)
			{
				for(int x = 0; x <= (this.Grid as GridControl).ColCount; x++)
					(this.Grid as GridControl).ColWidths.ResizeToFit(Syncfusion.Windows.Forms.Grid.GridRangeInfo.Col(x), Syncfusion.Windows.Forms.Grid.GridResizeToFitOptions.IncludeHeaders);
			}
            
		}


		private void user_Cancel(object sender, EventArgs e)
		{
			this.Grid.CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Canceled);
		}
	
		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			base.OnInitialize (rowIndex, colIndex);
			UpdateString(rowIndex, colIndex);
		}
	
		protected override bool OnSaveChanges()
		{
			string s = this.ddUser.GetValuesToString();
			this.Grid.Model[this.RowIndex, this.ColIndex].Text = s;
			return true;
		}

		public void UpdateString(int rowIndex, int colIndex)
		{
			ddUser.SetValuesFromString(this.Grid.Model[rowIndex, colIndex].Text);
		}
	}
}
